<?php

// This file is part of CertificatesWall module for Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * CertificatesWall module core interaction API
 *
 * @package    mod
 * @subpackage certificateswall
 * @copyright  Roman Gelembjuk
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once($CFG->dirroot.'/course/lib.php');
require_once($CFG->dirroot.'/grade/lib.php');
require_once($CFG->dirroot.'/grade/querylib.php');

define('_CERTIFICATESWALL_ENDPOINT','http://certificateswall.com/');
define('_CERTIFICATESWALL_API_LINK',_CERTIFICATESWALL_ENDPOINT.'?page=api&');

/**
 * Add certificate instance.
 *
 * @param stdClass $certificate
 * @return int new certificate instance id
 */
function certificateswall_add_instance($certificate) {
    global $DB;

    $certificate->timemodified = time();

    if ($returnid = $DB->insert_record('certificateswall', $certificate)) {
        $certificate->id = $returnid;

        $event = NULL;
        $event->name        = $certificate->name;
        $event->description = '';
        $event->courseid    = $certificate->course;
        $event->groupid     = 0;
        $event->userid      = 0;
        $event->eventtype   = 'course';
        $event->modulename  = 'certificate';
        $event->instance    = $returnid;

        add_event($event);
    }

    return $returnid;
}

/**
 * Update certificate instance.
 *
 * @param stdClass $certificate
 * @return bool true
 */
function certificateswall_update_instance($certificate) {
    global $DB;

    $certificate->timemodified = time();
    $certificate->id = $certificate->instance;

    if ($returnid = $DB->update_record('certificateswall', $certificate)) {

        if ($event->id = $DB->get_field('event', 'id', array('modulename'=> 'certificateswall', 'instance'=> $certificate->id))) {
            $event->name        = $certificate->name;
            update_event($event);
        } else {
            $event = NULL;
            $event->name        = $certificate->name;
            $event->description = '';
            $event->courseid    = $certificate->course;
            $event->groupid     = 0;
            $event->userid      = 0;
            $event->modulename  = 'certificateswall';
            $event->instance    = $certificate->id;

            add_event($event);
        }
    } else {
        $DB->delete_records('event', array('modulename'=>'certificateswall', 'instance'=> $certificate->id));
    }

    return $returnid;
}

/**
 * Given an ID of an instance of this module,
 * this function will permanently delete the instance
 * and any data that depends on it.
 *
 * @param int $id
 * @return bool success
 */
function certificateswall_delete_instance($id) {
    global $CFG, $DB, $USER;

    if (!$certificate = $DB->get_record('certificateswall', array('id'=> $id))) {
        return false;
    }

    // Prepare file record object
    if (!$cm = get_coursemodule_from_instance('certificateswall', $id)) {
        return false;
    }

    $result = true;

    $DB->delete_records('certificateswall_issues', array('certificateid'=> $id));

    if (!$DB->delete_records('certificateswall', array('id'=> $id))) {
        $result = false;
    }

    $context = get_context_instance(CONTEXT_MODULE, $cm->id);

    $fs = get_file_storage();

    $fs->delete_area_files($context->id);

    return $result;
}

/**
 * This function is used by the reset_course_userdata function in moodlelib.
 * This function will remove all posts from the specified certificate
 * and clean up any related data.
 * 
 * Written by Jean-Michel Vedrine
 * 
 * @param $data the data submitted from the reset course.
 * @return array status array
 */
function certificateswall_reset_userdata($data) {
    global $CFG, $DB;

    $componentstr = get_string('modulenameplural', 'certificateswall');
    $status = array();

    if (!empty($data->reset_certificate)) {
        $certificatessql = "SELECT cert.id " .
                           "FROM {certificateswall} cert " .
                           "WHERE cert.course = ?";

        $DB->delete_records_select('certificateswall_issues', "certificateid IN ($certificatessql)", array($data->courseid));
        $status[] = array('component'=>$componentstr, 'item'=>get_string('certificateremoved', 'certificateswall'), 'error'=>false);
    }

    // Updating dates - shift may be negative too
    if ($data->timeshift) {
        shift_course_mod_dates('certificateswall', array('timeopen', 'timeclose'), $data->timeshift, $data->courseid);
        $status[] = array('component'=>$componentstr, 'item'=>get_string('datechanged'), 'error'=>false);
    }

    return $status;
}

/**
 * Implementation of the function for printing the form elements that control
 * whether the course reset functionality affects the certificate.
 * 
 * Written by Jean-Michel Vedrine
 * 
 * @param $mform form passed by reference 
 */
function certificateswall_reset_course_form_definition(&$mform) {
    $mform->addElement('header', 'certificateheader', get_string('modulenameplural', 'certificateswall'));
    $mform->addElement('advcheckbox', 'reset_certificate', get_string('deletissuedcertificates','certificateswall'));
}

/**
 * Course reset form defaults.
 * 
 * Written by Jean-Michel Vedrine
 * 
 * @param $course
 * @return array
 */
function certificateswall_reset_course_form_defaults($course) {
    return array('reset_certificate'=>1);
}

/**
 * Returns information about received certificate.
 * Used for user activity reports.
 * 
 * @param stdClass $course
 * @param stdClass $user
 * @param stdClass $mod
 * @param stdClass $certificate
 * @return object|null
 */
function certificateswall_user_outline($course, $user, $mod, $certificate) {
    global $DB;

    if ($issue = $DB->get_record('certificateswall_issues', array('certificateid' => $certificate->id, 'userid' => $user->id))) {
        $result->info = get_string('issued', 'certificateswall');
        $result->time = $issue->certdate;
    } else {
        $result->info = get_string('notissued', 'certificateswall');
    }

    return $result;
}

/**
 * Returns information about received certificate.
 * Used for user activity reports.
 * 
 * @param stdClass $course
 * @param stdClass $user
 * @param stdClass $mod
 * @param stdClass $page
 * @return object|null
 */
function certificateswall_user_complete($course, $user, $mod, $certificate) {
   global $DB;

   if ($issue = $DB->get_record('certificateswall_issues', array('certificateid' => $certificate->id, 'userid' => $user->id))) {
        print_simple_box_start();
        echo get_string('issued', 'certificateswall').": ";
        echo userdate($issue->certdate);
        //certificateswall_print_user_files($certificate->id, $user->id);
        echo '<br />';
        print_simple_box_end();
    } else {
        print_string('notissuedyet', 'certificateswall');
    }
}

/**
 * Must return an array of user records (all data) who are participants
 * for a given instance of certificate.
 * 
 * @param int $certificateid
 * @return object list of participants
 */
function certificateswall_get_participants($certificateid) {
    global $CFG, $DB;

    //Get students
    $participants = $DB->get_records_sql("SELECT DISTINCT u.id, u.id
                                          FROM {user} u,
                                               {certificateswall_issues} a
                                          WHERE a.certificateid = '$certificateid' and
                                                u.id = a.userid");
    return $participants;
}

/**
 * @uses FEATURE_GROUPS
 * @uses FEATURE_GROUPINGS
 * @uses FEATURE_GROUPMEMBERSONLY
 * @uses FEATURE_MOD_INTRO
 * @uses FEATURE_COMPLETION_TRACKS_VIEWS
 * @uses FEATURE_GRADE_HAS_GRADE
 * @uses FEATURE_GRADE_OUTCOMES
 * @param string $feature FEATURE_xx constant for requested feature
 * @return mixed True if module supports feature, null if doesn't know
 */
function certificateswall_supports($feature) {
    switch($feature) {
        case FEATURE_GROUPS:                  return true;
        case FEATURE_GROUPINGS:               return true;
        case FEATURE_GROUPMEMBERSONLY:        return true;
        case FEATURE_MOD_INTRO:               return true;
        case FEATURE_COMPLETION_TRACKS_VIEWS: return true;
        case FEATURE_BACKUP_MOODLE2:          return true;

        default: return null;
    }
}

/**
 * Prints the header in view.  Used to help prevent FPDF header errors.
 * 
 * @param stdClass $course
 * @param stdClass $certificate
 * @param stdClass $cm
 * @return null
 */
function certificateswall_view_header($course, $certificate, $cm) {
    global $CFG, $DB, $PAGE, $OUTPUT;

    $PAGE->set_title(format_string($certificate->name));
    $PAGE->set_heading(format_string($course->fullname));
    echo $OUTPUT->header();

    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
    if (has_capability('mod/certificateswall:manage', $context)) {
        $numusers = count(certificateswall_get_issues($certificate->id, '', '', $cm));
        echo '<div class="reportlink"><a href="report.php?id='.$cm->id.'">'.
              get_string('viewcertificateviews', 'certificateswall', $numusers).'</a></div>';
    }

    if (!empty($certificate->intro)) {
        global $OUTPUT;

        echo $OUTPUT->box(format_text($certificate->intro), 'generalbox', 'intro');
    }
}

/**
 * Function to be run periodically according to the moodle cron
 * TODO:This needs to be done                        
 */
function certificateswall_cron () {
    return true;
}

/**
 * Returns a list of teachers by group                
 * for sending email alerts to teachers              
 * 
 * @param stdClass $certificate
 * @param stdClass $user
 * @param stdClass $course
 * @param stdClass $cm
 * @return array                                     
 */
function certificateswall_get_teachers($certificate, $user, $course, $cm) {
    global $USER, $DB;

    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
    $potteachers = get_users_by_capability($context, 'mod/certificateswall:manage', '', '', '', '', '', '', false, false);
    if (empty($potteachers)) {
        return array();
    }
    $teachers = array();
    if (groups_get_activity_groupmode($cm, $course) == SEPARATEGROUPS) {   // Separate groups are being used
        if ($groups = groups_get_all_groups($course->id, $user->id)) {  // Try to find all groups
            foreach ($groups as $group) {
                foreach ($potteachers as $t) {
                    if ($t->id == $user->id) {
                        continue; // do not send self
                    }
                    if (groups_is_member($group->id, $t->id)) {
                        $teachers[$t->id] = $t;
                    }
                }
            }
        } else {
            // user not in group, try to find teachers without group
            foreach ($potteachers as $t) {
                if ($t->id == $USER->id) {
                    continue; // do not send self
                }
                if (!groups_get_all_groups($course->id, $t->id)) { //ugly hack
                    $teachers[$t->id] = $t;
                }
            }
        }
    } else {
        foreach ($potteachers as $t) {
            if ($t->id == $USER->id) {
                continue; // do not send self
            }
            $teachers[$t->id] = $t;
        }
    }

    return $teachers;
}

/**
 * Alerts teachers by email of received certificates. First checks
 * whether the option to email teachers is set for this certificate.
 * 
 * @param stdClass $course
 * @param stdClass $certificate
 * @param stdClass $certrecord
 * @param stdClass $cm course module
 * @return null
 */
function certificateswall_email_teachers($course, $certificate, $certrecord, $cm) {
    global $USER, $CFG, $DB;

    if ($certificate->emailteachers == 0) {          // No need to do anything
        return;
    }
    $student = $certrecord->studentname;
    $user = $DB->get_record('user', array('id'=> $certrecord->userid));

    if ($teachers = certificateswall_get_teachers($certificate, $user, $course, $cm)) {
        $strawarded  = get_string('awarded', 'certificateswall');
        foreach ($teachers as $teacher) {
            unset($info);

            $info->student = $student;
            $info->course = format_string($course->fullname,true);
            $info->certificate = format_string($certificate->name,true);
            $info->url = $CFG->wwwroot.'/mod/certificateswall/report.php?id='.$cm->id;
            $from = $student;
            $postsubject = $strawarded.': '.$info->student.' -> '.$certificate->name;
            $posttext = certificateswall_email_teachers_text($info);
            $posthtml = ($teacher->mailformat == 1) ? certificateswall_email_teachers_html($info) : '';

            @email_to_user($teacher, $from, $postsubject, $posttext, $posthtml);  // If it fails, oh well, too bad.
        }
    }
}
/**
 * Alerts student by email of received certificates. 
 * 
 * @param stdClass $course
 * @param stdClass $certificate
 * @param stdClass $certrecord
 * @param stdClass $cm course module
 * @return null
 */
function certificateswall_email_student($course, $certificate, $certrecord, $cm,$user) {
    global $USER, $CFG, $DB,$SITE;

    if ($certificate->emailstudent == 0) {          // No need to do anything
        return;
    }
    $strawarded  = get_string('awarded', 'certificateswall');
    unset($info);

    $info->student = fullname($user);
    $info->course = format_string($course->fullname,true);
    $info->certificate = format_string($certificate->name,true);
    $info->url = $certrecord->cwlink;
    $from = $SITE->fullname;
    $postsubject = $strawarded.': '.$info->student.' -> '.$certificate->name;

    $posttext = get_string('emailstudentmail', 'certificateswall', $info)."\n";

    $posthtml = '';
    if($user->mailformat == 1){
    	    $posthtml  = '<font face="sans-serif">';
    	    $posthtml .= '<p>'.get_string('emailstudentmailhtml', 'certificateswall', $info).'</p>';
    	    $posthtml .= '</font>';
    }

    @email_to_user($user, $from, $postsubject, $posttext, $posthtml);  // If it fails, oh well, too bad.
}
/**
 * Alerts others by email of received certificates. First checks
 * whether the option to email others is set for this certificate.
 * Uses the email_teachers info.      
 * Code suggested by Eloy Lafuente 
 */
function certificateswall_email_others ($course, $certificate, $certrecord, $cm) {
    global $USER, $CFG, $DB;

    if ($certificate->emailothers) {
       $student = $certrecord->studentname;

       $others = explode(',', $certificate->emailothers);
        if ($others) {
            $strawarded  = get_string('awarded', 'certificateswall');
            foreach ($others as $other) {
                $other = trim($other);
                if (validate_email($other)) {
                    $destination->email = $other;
                    unset($info);
                    $info->student = $student;
                    $info->course = format_string($course->fullname,true);
                    $info->certificate = format_string($certificate->name,true);
                    $info->url = $CFG->wwwroot.'/mod/certificateswall/report.php?id='.$cm->id;
                    $from = $student;
                    $postsubject = $strawarded.': '.$info->student.' -> '.$certificate->name;
                    $posttext = certificateswall_email_teachers_text($info);
                    $posthtml = certificateswall_email_teachers_html($info);

                    @email_to_user($destination, $from, $postsubject, $posttext, $posthtml);  // If it fails, oh well, too bad.
                }
            }
        }
    }
}

/**
 * Creates the text content for emails to teachers -- needs to be finished with cron
 * 
 * @param $info object The info used by the 'emailteachermail' language string
 * @return string                                    
 */
function certificateswall_email_teachers_text($info) {
    $posttext = get_string('emailteachermail', 'certificateswall', $info)."\n";

    return $posttext;
}

/**
 * Creates the html content for emails to teachers
 * 
 * @param $info object The info used by the 'emailteachermailhtml' language string
 * @return string                                    
 */
function certificateswall_email_teachers_html($info) {
    $posthtml  = '<font face="sans-serif">';
    $posthtml .= '<p>'.get_string('emailteachermailhtml', 'certificateswall', $info).'</p>';
    $posthtml .= '</font>';

    return $posthtml;
}

/**
 * Retrieve certificate path from hash
 * 
 * @param array $contenthash
 * @return string the path
 */
function certificateswall_path_from_hash($contenthash) {
    $l1 = $contenthash[0].$contenthash[1];
    $l2 = $contenthash[2].$contenthash[3];
    return "$l1/$l2";
}


/**
 * Returns a list of issued certificates - sorted for report.
 * 
 * @param stdClass $certificate
 * @param string $sort the sort order
 * @param boolean $groupmode are we in group mode ?
 * @param stdClass $cm the course module
 */
function certificateswall_get_issues($certificate, $sort="u.studentname ASC", $groupmode, $cm) {
    global $CFG, $DB;
    // get all users that can manage this certificate to exclude them from the report.
    $context = get_context_instance(CONTEXT_MODULE, $cm->id);
    $certmanagers = get_users_by_capability($context, 'mod/certificateswall:manage', 'u.id');

    // get all the users that have certificates issued.
    $users = $DB->get_records_sql("SELECT u.*,u.picture, s.code, s.timecreated, s.certdate, s.studentname, s.reportgrade,s.cwlink,s.id as issueid
                                   FROM {$CFG->prefix}certificateswall_issues s,
                                        {$CFG->prefix}user u
                                   WHERE s.certificateid = '$certificate'
                                   AND s.userid = u.id
                                   AND s.certdate > 0
                                   GROUP BY u.id");
    // now exclude all the certmanagers.
    foreach ($users as $id => $user) {
        if (isset($certmanagers[$id])) { //exclude certmanagers.
            unset($users[$id]);
        }
    }

    // if groupmembersonly used, remove users who are not in any group
    if (!empty($users) and !empty($CFG->enablegroupings) and $cm->groupmembersonly) {
        if ($groupingusers = groups_get_grouping_members($cm->groupingid, 'u.id', 'u.id')) {
            $users = array_intersect($users, array_keys($groupingusers));
        }
    }

    if (!$groupmode) {
        return $users;
    } else {
        $currentgroup = groups_get_activity_group($cm);
        if ($currentgroup) {
            $groupusers = groups_get_members($currentgroup, 'u.*');
            if (empty($groupusers)) {
                return array();
            }
            foreach($groupusers as $id => $gpuser) {
                if (!isset($users[$id])) {
                    //remove this user as it isn't in the group!
                    unset($users[$id]);
                }
            }
        }

        return $users;
    }
}

/**
 * Returns a list of all course students or all who don't have a certificate.
 * 
 * @param stdClass $certificate
 * @param stdClass $cm the course module
 */
function certificateswall_get_course_students($certificate,$cm,$all=false,$groupmode=false){
	global $CFG, $DB;
	// get all users that can manage this certificate to exclude them from the report.
	$context = get_context_instance(CONTEXT_MODULE, $cm->id);
	$certmanagers = get_users_by_capability($context, 'mod/certificateswall:manage', 'u.id');
	$managers=array_keys($certmanagers);

	$sql="SELECT u.* ".
		"FROM {$CFG->prefix}user u JOIN (".
			"SELECT DISTINCT eu1_u.id FROM {$CFG->prefix}user eu1_u JOIN {$CFG->prefix}user_enrolments eu1_ue ON eu1_ue.userid = eu1_u.id JOIN {$CFG->prefix}enrol eu1_e ".
			"ON (eu1_e.id = eu1_ue.enrolid AND eu1_e.courseid = $certificate->course) WHERE eu1_u.deleted = 0  ) e ".
			"ON e.id = u.id ";
	$wh=array();
	if(count($managers)>0)
		$wh[]="u.id NOT IN (".join(',',$managers).")";
	
	if(!$all){
		//get students with certificates
		$l=certificateswall_get_issues($certificate->id, null, null, $cm);
		$issids=array_keys($l);
		if(count($issids)>0) $wh[]="u.id NOT IN (".join(',',$issids).")";
	}
	
	if(count($wh)>0) $sql.="WHERE ".join(' AND ',$wh); 
	
	$users = $DB->get_records_sql($sql);
	
	if (!empty($users) and !empty($CFG->enablegroupings) and $cm->groupmembersonly) {
		if ($groupingusers = groups_get_grouping_members($cm->groupingid, 'u.id', 'u.id')) {
			$users = array_intersect($users, array_keys($groupingusers));
		}
	}

	if (!$groupmode) {
		return $users;
	} else {
		$currentgroup = groups_get_activity_group($cm);
		if ($currentgroup) {
			$groupusers = groups_get_members($currentgroup, 'u.*');
			if (empty($groupusers)) {
				return array();
			}
			foreach($groupusers as $id => $gpuser) {
				if (!isset($users[$id])) {
					//remove this user as it isn't in the group!
					unset($users[$id]);
				}
			}
		}
		return $users;
	}
}
/**
 * Inserts preliminary user data when a certificate is viewed.
 * Prevents form from issuing a certificate upon browser refresh.
 * 
 * @param stdClass $course
 * @param stdClass $user
 * @param stdClass $certificate
 * @return null
 */
function certificateswall_prepare_issue($course, $user, $certificate,$cwcertid,$cwuserid) {
    global $DB;

    if ($DB->record_exists('certificateswall_issues', array('certificateid'=>$certificate->id, 'userid'=>$user->id))) {
           return;
    }

    $timecreated = time();
    $code = certificateswall_generate_code($certificate,$course,$user);
    $studentname = fullname($user);
    $newrec = new Object();
    $newrec->certificateid = $certificate->id;
    $newrec->userid = $user->id;
    $newrec->timecreated = $timecreated;
    $newrec->studentname = $studentname;
    $newrec->code = $code;
    $newrec->classname = $course->fullname;
    $newrec->cwcertid = $cwcertid;
    $newrec->cwuserid = $cwuserid;
    $newrec->cwlink='';

    $DB->insert_record('certificateswall_issues', $newrec, false);
    
    //increase last number for course.
    $certificate->lastnumber=$certificate->lastnumber+1;
    $DB->update_record('certificateswall', $certificate);
}

/**
 * Inserts final user data when a certificate is created.
 * 
 * @param stdClass $course
 * @param stdClass $certrecord
 * @param stdClass $cm
 */
function certificateswall_issue($course, $certrecord, $cm,$user=null,$autoissue=false) {
	global $USER, $DB, $certificate;
	
	if(!$user) $user=$USER;

	$certrecord = $DB->get_record('certificateswall_issues', array('certificateid'=>$certificate->id, 'userid'=>$user->id));
	
	if(!$certrecord) return false;
	
	$grade = certificateswall_print_course_grade($course,$user);

        $certrecord->reportgrade = $grade->percentage;

        $date = certificateswall_generate_date($certificate, $course);
        $certrecord->certdate = $date;

        $DB->update_record('certificateswall_issues', $certrecord);
        
        if($autoissue){
        	certificateswall_email_teachers($course, $certificate, $certrecord, $cm);
        	certificateswall_email_others($course, $certificate, $certrecord, $cm);
        }
}

/**
 * Inserts data on certificateswall.com side
 * 
 * @param stdClass $certrecord
 */
function certificateswall_issue_update($certrecord, $cert,$course=null,$cm=null,$user=null,$autoissue=true) {
	global $USER, $DB, $certificate;

        $certrecord->cwcertid = $cert->id;
        $certrecord->cwuserid = $cert->userid;
        $certrecord->cwlink = $cert->publiclink;
        $DB->update_record('certificateswall_issues', $certrecord);
        
        if($certificate->emailstudent && !$autoissue){
        	certificateswall_email_student($course, $certificate, $certrecord, $cm,$user);
        }

}
/**
 * Deletes issued certificate from DB
 * 
 * @param stdClass $issueid
 */
function certificateswall_issue_delete($issueid) {
	global $DB,$CFG;
	//load the isseu to get information
	$certrecord = $DB->get_record('certificateswall_issues', array('id'=>$issueid));
	
	if(!isset($certrecord) || $certrecord->id==0) return false;
	
	$user=$DB->get_record('user', array('id'=>$certrecord->userid));
	
	certificateswall_api_removecertificate($certrecord,$user);

	$DB->delete_records('certificateswall_issues', array('id'=>$issueid));
	
	return true;
}
/**
 * Used for course participation report (in case certificate is added).
 * 
 * @return array
 */
function certificateswall_get_view_actions() {
    return array('view','view all','view report');
}

/**
 * Used for course participation report (in case certificate is added).
 * 
 * @return array
 */
function certificateswall_get_post_actions() {
    return array('received');
}


/**
 * Prepare to print the course grade.
 * 
 * @param stdClass $course
 * @return mixed
 */
function certificateswall_print_course_grade($course,$user=null){
    global $USER, $DB;
    if(!$user) $user=$USER;
    if ($course_item = grade_item::fetch_course_item($course->id)) {
    	    
        $grade = new grade_grade(array('itemid'=>$course_item->id, 'userid'=>$user->id));
        
        $course_item->gradetype = GRADE_TYPE_VALUE;
        
        $coursegrade->points = grade_format_gradevalue($grade->finalgrade, $course_item, true, GRADE_DISPLAY_TYPE_REAL, $decimals=2);
        
        $coursegrade->percentage = grade_format_gradevalue($grade->finalgrade, $course_item, true, GRADE_DISPLAY_TYPE_PERCENTAGE, $decimals=2);
        
        $coursegrade->letter = grade_format_gradevalue($grade->finalgrade, $course_item, true, GRADE_DISPLAY_TYPE_LETTER, $decimals=0);

        return $coursegrade;
    }

    return false;
}
/**
 * Prepare to be print the date -- defaults to time.
 * 
 * @param stdClass $certificate
 * @param stdClass $course
 * @return string the date
 */
function certificateswall_generate_date($certificate, $course) {
	return time();
}
/**
 * Generates a 10-digit code of random letters and numbers.
 * 
 * @return string
 */
function certificateswall_generate_code($certificate=null,$course=null,$user=null) {
	if(isset($certificate) && isset($course) && isset($user) && $certificate->certificateidformat==1 && $certificate->certificatecustomformat!=''){
		if(strpos($certificate->certificatecustomformat,'%')!==FALSE){
			//generate code
			//Y - current year,<br>M - current month,<br>D - current day,<br>S - student ID in system,<br>C - course ID,<br>N - number of certificate in course.
			$code=$certificate->certificatecustomformat;
			$code=preg_replace('!%Y!',date('Y'),$code);
			$code=preg_replace('!%M!',date('m'),$code);
			$code=preg_replace('!%D!',date('d'),$code);
			$code=preg_replace('!%S!',$user->id,$code);
			$code=preg_replace('!%C!',$course->id,$code);
			$code=preg_replace('!%N!',($certificate->lastnumber+1),$code);
			return $code;
		}
	}
	return (random_string(10));
}

function certificateswall_get_link_for_iframe($certrecord){
	if($certrecord->cwcertid>0 && $certrecord->cwuserid>0){
		$link=_CERTIFICATESWALL_ENDPOINT.'index.php?page=public&id='.$certrecord->cwuserid.'&certificate='.$certrecord->cwcertid.'&tmpl=lempty&onlycertificate=y';
		return $link;
	}
	return '';
}
function certificateswall_get_remote_information($certificate,$course){
	global $CFG;
	if($certificate->certificateinfo!=''){
		$info=json_decode($certificate->certificateinfo);
		if($info->apikey==$CFG->certificateswall_apikey && isset($info->lastrequesttime) && time()-$info->lastrequesttime>3600*24){
			return $info;
		}
	}

	$info=certificateswall_api_get_info($course,$certificate->certificatetemplate);

	$info=$info->response->info;
	$info->demoimages->big=_CERTIFICATESWALL_ENDPOINT.$info->demoimages->path.$info->demoimages->big;
	$info->demoimages->middle=_CERTIFICATESWALL_ENDPOINT.$info->demoimages->path.$info->demoimages->middle;
	//$info->demoimages->small=_CERTIFICATESWALL_ENDPOINT.$info->demoimages->path.$info->demoimages->small;
	$info->apikey=$CFG->certificateswall_apikey;
	$info->lastrequesttime=time();

	$str=json_encode($info);
	$certificate->certificateinfo=$str;
	$certificate->instance=$certificate->id;
	certificateswall_update_instance($certificate);
	return $info;
}
//========= API functions =========================================
function certificateswall_api_execute($command,$params='',$course=null) {
	global $CFG,$SITE;
	if($CFG->certificateswall_function=='demo') $CFG->certificateswall_apikey='';
	$url=_CERTIFICATESWALL_API_LINK.'action='.$command.'&apikey='.urlencode($CFG->certificateswall_apikey);

	if($CFG->certificateswall_function=='demo' && isset($course)){
		$url.='&demoagency=moodlebase&thissite='.urlencode($CFG->wwwroot).
			'&sitename='.urlencode($SITE->fullname).'&shortsitename='.urlencode($SITE->shortname).
			'&coursename='.urlencode($course->fullname).'&courseshortname='.urlencode($course->shortname);
	}
	
	$url.='&'.$params;
	$agent = "Moodle plugin from ".$_SERVER["SERVER_NAME"];

	if(function_exists('curl_init')){
		  $ch = curl_init();
		  curl_setopt($ch, CURLOPT_URL, $url);
		  curl_setopt($ch, CURLOPT_USERAGENT, $agent);
		  curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		  curl_setopt($ch, CURLOPT_TIMEOUT, 20);
		  curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);

		  $result = curl_exec ($ch);

		  if (curl_error($ch)) { 
			      $result="";
		  } 

		curl_close ($ch);
	}else{
		  $result=file_get_contents($url);
	}
	return json_decode($result);
}
function certificateswall_api_createuser($user,$authorise='no',$course=null) {
	
	$studentname = fullname($user);
	return certificateswall_api_execute('createuser','email='.urlencode($user->email).
		'&name='.urlencode($studentname).'&username='.$user->username.'&sendauthemail='.$authorise,$course);
}

function certificateswall_api_addcertificate($certrecord,$user,$course=null) {
	global $DB,$CFG;
	$context = get_context_instance(CONTEXT_MODULE, $certrecord->certificateid);

	$teachername='';
	$teacherslist=array();

	if($context){
	if ($teachers = get_users_by_capability($context, 'mod/certificateswall:printteacher', '', $sort='u.lastname ASC','','','','',false)) {
		foreach ($teachers as $teacher) {
			$teacherslist[]=fullname($teacher);
			if($teachername=='') $teachername=fullname($teacher);
		}
	}
	}

	$certificate = $DB->get_record('certificateswall', array('id'=> $certrecord->certificateid));
	$enddate='';
	
	$dateformat=($CFG->certificateswall_dateformat!='')?$CFG->certificateswall_dateformat:'F d, Y';
	if($certificate->certificateenddate!=''){
		$enddate=$certificate->certificateenddate;
	}elseif(intval($certificate->certificatetime)>0){
		$enddate=date($dateformat,strtotime(date($dateformat,$certrecord->certdate).' + '.$certificate->certificatetime.' months'));
	}
	
	$coursedate=date($dateformat,$certrecord->certdate);
	if($certificate->certificateassigndate!='') $coursedate=$certificate->certificateassigndate;

	return certificateswall_api_execute('addcertificate',
		'name='.urlencode($certrecord->studentname).
		'&templateid='.urlencode($certificate->certificatetemplate).
		'&date='.urlencode($coursedate).
		'&program='.urlencode($certrecord->classname).
		'&programshort='.urlencode($course->shortname).
		'&coursehours='.urlencode($certificate->coursehours).
		'&enddate='.urlencode($enddate).
		'&code='.urlencode($certrecord->code).
		'&email='.urlencode($user->email).
		'&grade='.urlencode($certrecord->reportgrade).
		'&teacher='.urlencode($teachername).
		'&allteachers='.urlencode(join(';',$teacherslist)),
		$course
		);
}

function certificateswall_api_removecertificate($certrecord,$user) {

	return certificateswall_api_execute('removecertificate',
		'code='.urlencode($certrecord->code).
		'&email='.urlencode($user->email));
}

function certificateswall_api_get_info($course,$templateid=0) {
	global $CFG,$SITE;
	return certificateswall_api_execute('getaccountinfo','&program='.urlencode($course->fullname).'&programshort='.urlencode($course->shortname).
		'&vendor='.urlencode($SITE->fullname).'&templateid='.$templateid,$course);
}

function certificateswall_api_get_templates() {
	global $CFG,$SITE;
	return certificateswall_api_execute('gettemplates','');
}

function certificateswall_api_get_print_url($certrecord,$user,$course) {
	return certificateswall_api_execute('getprintablecertificate','certificateid='.$certrecord->cwcertid.'&email='.urlencode($user->email),$course);
}